/*
 * Matrix.h
 *
 *  Created on: 21.3.2011
 *      Author: hartman
 */

#ifndef MATRIX_H_
#define MATRIX_H_
#include <string>
#include <sstream>
#include <iostream>
#include <math.h>
#include "StringOperations.h"

template <class T>
class SymMatrix {
public:
	SymMatrix();
	SymMatrix(SymMatrix<T> *lsymmat);
	SymMatrix(T* lmat, int lsize, bool triu, bool absolute);
	SymMatrix(int lsize, T* initElem);
	bool specified();
	virtual ~SymMatrix();
	T& operator[] (int i) const;
	T operator() (int i, int j) const;
	T sumOfData();
	T maxOfData();
	T* getData();
	T* getFullData();
	int size();
	int len();
	SymMatrix<bool>* binarize(T t);
	std::string to_s();

private:
	T* mat;
	int dim;
	int num;

	bool dataGet;

	std::string to_s(T number);
	void initialSetting();
};


template <typename T> SymMatrix<T>::SymMatrix(SymMatrix<T> *lsymmat) {
	mat = lsymmat->mat;
	dim = lsymmat->dim;
	num = lsymmat->num;
	initialSetting();
}

template <typename T> SymMatrix<T>::SymMatrix() {
	mat = NULL;
	dim = 0;
	num = 0;
	initialSetting();
}

template <typename T> SymMatrix<T>::SymMatrix(int lsize, T* initElem)
{
	dim = lsize;
	num = lsize * (lsize + 1) / 2;
	mat = new T[num];
	if (initElem != NULL)
	{
		int c = *initElem;
		for (int i = 0; i < num; i++)
			mat[i] = c;
	}
	initialSetting();
}

template <typename T> SymMatrix<T>::SymMatrix(T* lmat, int lsize, bool triu, bool absolute) {

	dim = lsize;
	num = lsize * (lsize + 1) / 2;

	if (triu)
	{
		mat = lmat;
		if (absolute)
			for (int i = 0; i < num; i++)
				mat[i] = fabs(mat[i]);
	}
	else
	{
		mat = new T[num];
		int indx = 0;
		for (int i = 0; i < lsize; i++)
			for (int j = i; j < lsize; j++)
				mat[indx++] = (absolute) ? fabs(lmat[i*lsize + j]):lmat[i*lsize + j];
	}
	initialSetting();
}

template <typename T> void SymMatrix<T>::initialSetting()
{
	dataGet = false;
}

template <typename T> bool SymMatrix<T>::specified()
{
	return (mat != NULL);
}


template <typename T> SymMatrix<T>::~SymMatrix() {
	if (mat != NULL && !dataGet)
		delete [] mat;
}

template <typename T> T& SymMatrix<T>::operator[] (int i) const
{
	return mat[i];
}


template <typename T> T SymMatrix<T>::operator() (int i, int j) const
{
	return (i <= j) ? (mat[i*dim - (i*(i+1)/2) + j]):(mat[j*dim - (j*(j+1)/2) + i]);
}

template <typename T> T SymMatrix<T>::sumOfData()
{
	T sum = 0;
	for (int i = 0; i < num; i++)
		sum = sum + mat[i];
	return sum;
}

template <typename T> T SymMatrix<T>::maxOfData()
{
	T dmax = mat[0];
	for (int i = 1; i < num; i++)
		if (dmax < mat[i])
			dmax = mat[i];

	return dmax;
}


template <typename T> T* SymMatrix<T>::getData()
{
	dataGet = true;
	return mat;
}

template <typename T> T* SymMatrix<T>::getFullData()
{
	T* dat = new T[dim*dim];
	for (int i = 0; i < dim; i++)
		for (int j = 0; j < dim; j++)
			dat[i*dim + j] = (*this)(i,j);

	return dat;
}

template <typename T> int SymMatrix<T>::size()
{
	return dim;
}

template <typename T> int SymMatrix<T>::len()
{
	return num;
}

template <typename T> SymMatrix<bool>* SymMatrix<T>::binarize(T t)
{
	bool* binMat = new bool[num];
	for (int i = 0; i < num; i++)
		binMat[i] = (mat[i] > t) ? 1:0;

	return new SymMatrix<bool>(binMat, dim, true, false);
}

template <typename T> std::string SymMatrix<T>::to_s()
{
	std::stringstream matstr;
	for (int i = 0; i < dim; i++)
	{
		for (int j = 0; j < dim; j++)
		{
			matstr << StringOperations::formatOutputNumber((*this)(i,j));
			matstr << "\t";
		}
		matstr << "\n";
	}
	return matstr.str();
}

#endif /* MATRIX_H_ */
