/*
 * NetworksFunctionsInterface.h
 *
 *  Created on: 19.5.2011
 *      Author: hartman
 */

#ifndef NETWORKSFUNCTIONSINTERFACE_H_
#define NETWORKSFUNCTIONSINTERFACE_H_
#include <vector>
#include "Graph.h"
#include "WeightedGraph.h"

WeightedGraph* loadWeightedGraph(double* mat, int size);
Graph* loadGraph(double* mat, int size, double threshold);

// betweenness
double* betweenneess_weighted(double* mat, int size);
double* betweenneess_binary(double* mat, int size, double threshold);
void betweenneess_binary_sparse(int* mat, int size, double* BC);

// components
double* components_binary(double* mat, int size, double threshold, int &numComp);
double* components_weighted(double* mat, int size, int &numComp);

// degrees
int* degrees_binary(double* mat, double threshold, int size);

// strengths
double* strengths_weighted(double* mat, int size);

// assortativity
double assortativity_binary(double* mat, int size, double threshold);

// charpath length
double charpath_len_binary(double* mat, int size, double threshold, int*& D);
double charpath_len_weighted(double* mat, int size, double*& D);
double charpath_len_dist_weighted(double* dmat, int size);
double charpath_len_dist_binary(int* dmat, int size);

// distance matrix
double efficiency_binary(double* mat, int size, double threshold, int*& D);
double efficiency_weighted(double* mat, int size, double*& D);
double efficiency_dist_weighted(double* dmat, int size);
double efficiency_dist_binary(int* dmat, int size);

// distance matrix
int* distance_mat_binary(double* mat, int size, double threshold);
double* distance_mat_weighted(double* mat, int size);

// delta betweenness - always weighted
double* delta_betweenness(double* mat, int size, double weightDiffEpsilon);

// eigenvector betweenness
//double* eigvec_betweenness_binary(double* mat, int size, double threshold);

// random walk betweenness
//double* randwalk_betweenness_binary(double* mat, int size, double threshold);

// closeness
double closeness_binary(double* mat, int size, double threshold);


// aux
std::vector<std::vector<int> >* components_binary_vec(double* mat, int size, double threshold);

#endif /* NETWORKSFUNCTIONSINTERFACE_H_ */
