/*
 * SparseGraph.cpp
 *
 *  Created on: 23.6.2011
 *      Author: hartman
 */

#include "SparseGraph.h"
#include <iostream>
#include <cstring>
#include <queue>
#include <vector>
#include <list>
#include <stack>

#define INDICATION false
#define DEBUG false


SparseGraph::SparseGraph(int* mat, int lsize) {
	size = lsize;

	neighs = new int*[size];
	degrees = new int[size];
	int* aux = new int[size];

	for (int i = 0; i < size; i++)
	{
		int idx = 0;
		int pos = i * size;
		for (int j = 0; j < size; j++)
			if (mat[pos + j] > 0.5)
				aux[idx++] = j;

		degrees[i] = idx;
		if (idx > 0)
		{
			neighs[i] = new int[idx];
			memcpy(neighs[i],aux,idx*sizeof(int));
		}
		else
		{
			neighs[i] = NULL;
		}
	}

	delete aux;
}

SparseGraph::~SparseGraph() {
	if (neighs != NULL)
		delete neighs;

}

string SparseGraph::to_s()
{
	//stringstream matstr(stringstream::in);
	for (int i = 0; i < size; i++)
	{
		int idx = 0;
		if (neighs[i] == NULL)
		{
			for (int j = 0; j < size; j++)
				cout << "0\t";
		}
		else
		{
			for (int j = 0; j < size; j++)
			{
				if (neighs[i][idx] == j)
				{
					cout << "1";
					idx++;
				}
				else
				{
					cout << "0";
				}
				cout << "\t";
			}
		}
		cout << "\n";
	}
	return " ";
}

double SparseGraph::betweeness(double* betweenessVector)
{
	for (int i = 0; i < size; i++)
		betweenessVector[i] = 0.0;

	unsigned long* sigma = new unsigned long[size];
	long* d = new long[size];
	double* delta = new double[size];
	list<int>* P = new list<int>[size];

	for (int s = 0; s < size; s++)
	{

		#if (INDICATION)
			cout << "\r Betweeness:" << s << "/" << size;
			cout.flush();
		#endif

		stack<int> S;

		for (int j = 0; j < size; j++)
		{
			sigma[j] = 0;
			d[j] = -1;
		}
		sigma[s] = 1;
		d[s] = 0;


		queue<int> Q;
		Q.push(s);  // maybe push_back

		while (!Q.empty())
		{
			int v = Q.front();
			Q.pop();
			S.push(v);

			// go through all neighbors
			int wdeg = degrees[v];
			long d_v = d[v];
			unsigned long sigma_v = sigma[v];
			for (int wid = 0; wid < wdeg; wid++)
			{
				int w = neighs[v][wid];
				if (d[w] < 0)
				{
					Q.push(w);
					d[w] = d_v + 1;
				}

				if (d[w] == d_v + 1)
				{
					sigma[w] = sigma[w] + sigma_v;
					P[w].push_back(v);
				}
			}
		}

		for (int j = 0; j < size; j++)
			delta[j] = 0.0;

		#if (DEBUG)
			cout << "Sigmas:" << endl;
			for (int j = 0; j < size; j++)
				cout << sigma[j] << " ";
			cout << endl;
		#endif


		while (!S.empty())
		{
			int w = S.top();
			S.pop();
			list<int>::iterator it;

			#if (DEBUG)
				cout << "Prvek:" << w << endl;
			#endif

			double wcoeff = (1.0+delta[w]) / ((double)sigma[w]);

			for (it = P[w].begin() ; it != P[w].end(); it++)
			{
				int v = *it;  // int v = (int) *it
				delta[v] = delta[v] + sigma[v]*wcoeff;

				#if (DEBUG)
					cout << "\t "<< delta[v] << "=" << sigma[v] << "/" << sigma[w] << "*(1 + " << delta[w] << ")" << endl;
					cout << "\t Cb = "<< betweenessVector[w] << endl;
				#endif
			}

			if (w != s)
			{
				betweenessVector[w] = betweenessVector[w] + delta[w];
			}
		}

		// delete aux data
		for (int i = 0;  i < size; i++)
			P[i].clear();
	}

	delete[] P;
	delete sigma;
	delete d;
	delete delta;

	double sum = 0.0;
	for (int i = 0; i < size; i++)
	{
		sum += betweenessVector[i];
	}

	double result = (1/((double) size))*sum;

	return result;
}

double SparseGraph::betweenessTest(double* betweenessVector)
{
	for (int i = 0; i < size; i++)
		betweenessVector[i] = 0.0;

	double* sigma = new double[size];
	long* d = new long[size];
	double* delta = new double[size];
	list<int>* P = new list<int>[size];

	for (int s = 0; s < size; s++)
	{

		#if (INDICATION)
			cout << "\r Betweeness:" << s << "/" << size;
			cout.flush();
		#endif

		stack<int> S;

		for (int j = 0; j < size; j++)
		{
			sigma[j] = 0.0;
			d[j] = -1;
		}
		sigma[s] = 1.0;
		d[s] = 0;


		queue<int> Q;
		Q.push(s);  // maybe push_back

		while (!Q.empty())
		{
			int v = Q.front();
			Q.pop();
			S.push(v);

			// go through all neighbors
			int wdeg = degrees[v];
			long d_v = d[v];
			double sigma_v = sigma[v];
			for (int wid = 0; wid < wdeg; wid++)
			{
				int w = neighs[v][wid];
				if (d[w] < 0)
				{
					Q.push(w);
					d[w] = d_v + 1;
				}

				if (d[w] == d_v + 1)
				{
					sigma[w] = sigma[w] + sigma_v;
					P[w].push_back(v);
				}
			}
		}

		for (int j = 0; j < size; j++)
			delta[j] = 0.0;

		#if (DEBUG)
			cout << "Sigmas:" << endl;
			for (int j = 0; j < size; j++)
				cout << sigma[j] << " ";
			cout << endl;
		#endif


		while (!S.empty())
		{
			int w = S.top();
			S.pop();
			list<int>::iterator it;

			#if (DEBUG)
				cout << "Prvek:" << w << endl;
			#endif

			double wcoeff = (1.0+delta[w]) / sigma[w];

			for (it = P[w].begin() ; it != P[w].end(); it++)
			{
				int v = *it;  // int v = (int) *it
				delta[v] = delta[v] + sigma[v]*wcoeff;

				#if (DEBUG)
					cout << "\t "<< delta[v] << "=" << sigma[v] << "/" << sigma[w] << "*(1 + " << delta[w] << ")" << endl;
					cout << "\t Cb = "<< betweenessVector[w] << endl;
				#endif
			}

			if (w != s)
			{
				betweenessVector[w] = betweenessVector[w] + delta[w];
			}
		}

		// delete aux data
		for (int i = 0;  i < size; i++)
			P[i].clear();
	}

	delete[] P;
	delete sigma;
	delete d;
	delete delta;

	double sum = 0.0;
	for (int i = 0; i < size; i++)
	{
		sum += betweenessVector[i];

	}

	double result = (1/((double) size))*sum;

	return result;
}

