/*
 * ConnectivityMatrixSource.cpp
 *
 *  Created on: 9.10.2009
 *      Author: hartman
 */

#include "WeightedGraph.h"
#include <stdio.h>
#include <iostream>
#include <fstream>
#include <algorithm>
#include <math.h>
#include <stdlib.h>
#include <limits>

using namespace std;

#define DEBUG false
#define INDICATION false


WeightedGraph::WeightedGraph(SymMatrix<double>* adjMat)
{
	// TODO one can delete it outside - think about it
	data = *adjMat;

	initialSetttings();

	#if (INDICATION)
	cout << "Data loaded!" << endl << data.to_s() << endl;
	#endif
}

WeightedGraph::WeightedGraph(SymMatrix<double>* mat,SymMatrix<double>* dmat)
{
	if (mat != NULL)
		data = *mat;

	initialSetttings();

	D = dmat;
}

WeightedGraph::~WeightedGraph() {
	if (!strengthsGet && strengthsData != NULL)
		delete strengthsData;
	if (!betweenessGet && betweenessVector != NULL)
		delete betweenessVector;
	if (!componentsGet && componentsList != NULL)
		delete componentsList;

}

void WeightedGraph::initialSetttings()
{
	// vector data
	clusteringVector = NULL;
	strengthsData = NULL;
	betweenessVector = NULL;
	deltaBetweenessVector = NULL;
	componentsList = NULL;

	// characteristics were get from outside - do not delete them
	strengthsGet = false;
	betweenessGet = false;
	deltaBetweenessGet = false;
	componentsGet = false;

	// distance matrix
	D = NULL;

	// some processing vars
	weightDiffEpsilon = 0.0;
	precisionEpsilon = 1e-10;
	sumweights = NULL;
	numedges = -1;
}

void WeightedGraph::resetValues()
{
	if (!strengthsGet && strengthsData != NULL)
		delete strengthsData;
	if (!betweenessGet && betweenessVector != NULL)
		delete betweenessVector;
	if (!deltaBetweenessGet && betweenessVector != NULL)
		delete deltaBetweenessVector;

	betweenessVector = NULL;
	deltaBetweenessVector = NULL;
	betweenessGet = false;
	deltaBetweenessGet = false;
}

void WeightedGraph::absoluteValues()
{
	for (int i = 0; i < data.len(); i++)
	{
		// TODO - probably assign operator
		//data[i] = fabs(data[i]);
	}
}


Graph* WeightedGraph::getUnweightedGraph(double threshold)
{
	return new Graph(data.binarize(threshold));
}

int WeightedGraph::getNumberOfEdges()
{
	if (numedges < 0)
	{
		numedges = 0;

		// can sum all the element of array twice (zero elements are counted like zero anyway)
		// assume all elements on diagonal is 0, so we can sum it all and then make it twice
		for (int i = 0; i < data.len(); i++)
		{
			numedges += (data[i] != 0);
		}
	}

	return numedges;
}

int WeightedGraph::getNumberOfEdges(double threshold)
{
	int thresholdEdges = 0;

	// can sum all the element of array twice (zero elements are counted like zero anyway)
	// assume all elements on diagonal is 0, so we can sum it all and then make it twice
	for (int i = 0; i < data.len(); i++)
	{
		if (data[i] > threshold)
			thresholdEdges++;
	}

	return thresholdEdges;
}

double WeightedGraph::sumOfWeigths()
{
	if (sumweights == NULL)
	{
		sumweights = new double[1];	*sumweights = 0.0;

		// can sum all the element of array (each edge gives one weights)
		for (int i = 0; i < data.len(); i++)
		{
			(*sumweights) = (*sumweights) + data[i];
		}
	}

	return *sumweights;
}

int WeightedGraph::getSize()
{
	return data.size();
}

double WeightedGraph::min()
{
	double result = data[0];
	for (int i = 1; i < data.len(); i++)
		if (data[i] < result)
			result = data[i];

	return result;
}

double WeightedGraph::max()
{
	double result = data[0];
	for (int i = 1; i < data.len(); i++)
		if (data[i] > result)
			result = data[i];

	return result;
}


double WeightedGraph::getDensity()
{
	if (density < NO_VALUE)
	{
		sumOfWeigths();

		double allEdges = (double) data.size()*(data.size() - 1) / 2.0;

		density = (*sumweights) / allEdges;
	}

	return density;
}

bool WeightedGraph::findNextMin(double value, double *restrictedMin)
{
	bool found = false;
	*restrictedMin = sumOfWeigths();
	for (int i = 0; i < data.len(); i++)
	{
		double d = data[i];
		if (d > value && d < *restrictedMin)
		{
			*restrictedMin = d;
			found = true;
		}
	}

	return found;
}

double* WeightedGraph::getComponentDataMat(int &numComp)
{
	if (componentsList == NULL)
	{
		components();
	}

	// data first dimension is threshold and second contain component matrix
	int compNum = componentsList->size();
	numComp = compNum;
	int numNodes = data.size();
	int clen = numNodes*compNum;
	// matrix holding vertices for all components
	double* componentsMat = new double[clen];

	// zero an array
	for (int j = 0; j < clen; j++)
		componentsMat[j] = 0.0;

	// iterate components
	for (int j = 0; j < compNum; j++)
	{
		for (size_t k = 0; k < (*componentsList)[j].size(); k++)
		{
			int numIdx = (*componentsList)[j][k];
			componentsMat[numIdx*compNum + j] = 1.0;
		}
	}

	return componentsMat;
}


vector<vector<int> >* WeightedGraph::getComponentsData()
{
	if (componentsList == NULL)
	{
		components();
	}

	componentsGet = true;

	return componentsList;
}


int WeightedGraph::components()
{
	componentsList = new vector<vector<int>  >();
	visited = new bool[data.size()];
	for (int i = 0; i < data.size(); i++)
		visited[i] = false;


	componentIndex = -1;
	for (int i = 0; i < data.size(); i++)
	{
		if (visited[i] == false)
		{
			componentIndex++;
			componentsList->push_back(vector<int>());

			visit(i);
		}
	}


	return componentIndex+1;
}

int WeightedGraph::visit(int i)
{
	visited[i] = true;
	(*componentsList)[componentIndex].push_back(i);
	for (int j = 0; j < data.size(); j++)
	{
		if (this->operator ()(i,j) && visited[j] == false)
		{
			visit(j);
		}
	}
	return 0;
}

vector<double> WeightedGraph::getEdges()
{
	// TODO effective is this not :-D
	double* mat = data.getData();
	vector<double> aux(data.len()-data.size(),0.0);
	int indx = 0, sindx = 1;
	for (int i = 0; i < data.size(); i++, sindx++)
	{
		for (int j = i+1; j < data.size(); j++)
		{
			aux[indx++] = mat[sindx++];
		}
	}

	return aux;
}

vector<double>* WeightedGraph::getThresholds(vector<double>* densities)
{
	// TODO densities should be sorted alterantively

	// get and copy data from matrix
	vector<double> aux(getEdges());
	// sort data (edges)
	sort(aux.begin(), aux.end());

//	cout << data.to_s() << endl;
//	cout << "Original: " << endl << StringOperations::triuStr(data.getData(), data.size()) << endl;
//	cout << "Result: " << endl << StringOperations::triuVecStr(aux, data.size()-1) << endl;

	// iterators
	vector<double>::iterator sit = aux.begin();
	vector<double>::iterator it;

	// thresholds results and index to densities == index to threshods
	vector<double>* thresholds = new vector<double>(densities->size(),0.0);
	size_t densityIdx = 0;

	// actual and minimal number of edges
	int numedges = data.size()*(data.size() - 1) / 2.0; // it has to be integer
	double maxNumedges = (double) numedges;

	// densities
	double d1 = 1.0;
	double d2 = 1.0;
	while (sit != aux.end())
	{
		// find interval of same numbers
		it = find_end(sit, aux.end(), sit, sit + 1);
		int sidx = int(sit-aux.begin());
		int eidx = int(it-aux.begin());

		// calculate new density
		numedges = numedges - (eidx - sidx + 1);
		d2 = (double) numedges / (double) maxNumedges;
		//cout << "Density: " << numedges << "/" << maxNumedges << " = "<< d2 << endl;

		if (d2 < (*densities)[densityIdx]) // between densities
		{
			// get threshold - TODO zero equality
			if (aux[sidx] == 0.0)
				(*thresholds)[densityIdx] = aux[sidx];
			else
				(*thresholds)[densityIdx] = (d1 - (*densities)[densityIdx] > (*densities)[densityIdx] - d2) ? aux[sidx]:aux[sidx-1];

			// shift to next
			densityIdx++;

			// if all thresholds are assigned, then end it
			if (densityIdx >= densities->size())
				break;

			// shift only if new density is smaller then alredy studied - otherwise there is a change for same threshold like before
			if (d2 > (*densities)[densityIdx])
				sit = it + 1;
		}
		else // larger than founded one
		{
			// automatic shift to next one
			sit = it + 1;
		}

		// old become new
		d1 = d2;
	}

//	for (int i = 0; i < densities->size(); i++)
//		cout << " (" << (*thresholds)[i] << " ~ " << (*densities)[i] << ") ";
//	cout << endl;

	return thresholds;
}

//int WeightedGraph::getThresholds(double t0, double* &thresholds, double* &densities, int *maxTresholdIndex)
//{
//	// size of arrays
//	int ressize = data.size()*(data.size() - 1) / 2;
//	double t = 0;
//	thresholds = new double[ressize];
//	densities = new double[ressize];
//
//	// actual density
//	double r = getUnweightedDensity(t);
//	thresholds[0] = t;
//	densities[0] = r;
//
//	// search part
//	double restMin;
//	int i = 0;
//	while (findNextMin(t,&restMin))
//	{
//		t = restMin;
//		r = getUnweightedDensity(t);
//		thresholds[i] = t;
//		densities[i] = r;
//		i++;
//	}
//
//	*maxTresholdIndex = i;
//	// for rest part - sometimes it can erase more egdes and there were some zeros on the beggining
//	// => fill rest of array with last values
//	t = thresholds[i-1];
//	r = densities[i-1];
//	for (int j = i; j < ressize; j++)
//	{
//		thresholds[j] = t;
//		densities[j] = r;
//	}
//
//	return ressize;
//}

double WeightedGraph::getUnweightedDensity()
{
	// \rho = |E|/(n \choose 2)

	numedges = getNumberOfEdges();

	double allEdges = (double) data.size()*(data.size() - 1) / 2.0;

	return ((double) numedges) / allEdges;
}

double WeightedGraph::getUnweightedDensity(double threshold)
{
	// \rho = |E|/(n \choose 2)

	int thresholdEdges = getNumberOfEdges(threshold);

	double allEdges = (double) data.size()*(data.size() - 1) / 2.0;

	double result = ((double) thresholdEdges) / allEdges;

	return result;
}

int WeightedGraph::generateRandomMatrix(string filename, int size, double min, double max)
{
	// generate and store each line separately
	double* row = new double[size];
	double interval = max - min;

	FILE* pFile;
	pFile = fopen (filename.c_str() , "wb" );

	if (pFile != NULL)
	{
		// write dimensions
		fwrite (&size , sizeof(size) , 1 , pFile);
		fwrite (&size , sizeof(size) , 1 , pFile);

		// generate and write matrix
		for (int j = 0; j < size; j++)
		{
			for (int i = j; i < size; i++)
			{
				double d;
				if (i == j)
					d = 0.0;
				else
					d = ((double) rand() / (double) RAND_MAX)*interval + min;
				row[i] = d;
			}

			fwrite (row , sizeof(double) , size , pFile);
		}
		fclose (pFile);
	}
	else {
		return -1;
	}

	return 0;
}

double WeightedGraph::averageStrength()
{
	if (strengthsData == NULL)
	{
		strengths();
	}

	long double sum = 0.0;
	for (int i = 0; i < data.size(); i++)
	{
		sum = sum + strengthsData[i];
	}

	long double res = sum / ((long double) data.size());

	return res;
}


double* WeightedGraph::strengths()
{
	if (strengthsData == NULL)
	{
		strengthsData = new double[data.size()];

		for (int i = 0; i < data.size(); i++)
		{
			strengthsData[i] = 0;
			for (int j = 0; j < data.size(); j++)
			{
				strengthsData[i] = strengthsData[i] + data(i,j);
			}
		}

	}

	strengthsGet = true;

	return strengthsData;
}

void WeightedGraph::setWeightDiffEpsilon(double aWeightDiffEpsilon)
{
	weightDiffEpsilon = aWeightDiffEpsilon;
}


double WeightedGraph::operator() (int i, int j)
{
	return data(i,j);
	// TODO erase
	//return (i <= j) ? (data[i*data.size() - (i*(i+1)/2) + j]):(data[j*data.size() - (j*(j+1)/2) + i]);
}

double WeightedGraph::dist(int i, int j)
{
	return (D == NULL) ? -1:((i <= j) ? ((*D)[i*data.size() - (i*(i+1)/2) + j]):((*D)[j*data.size() - (j*(j+1)/2) + i]));
}

int WeightedGraph::numerOfTriangles(int nodeIndex)
{
	// auxiliary array with one on neighbor positions
	char* nodeNeighs = new char[data.size()];
	for (int j = 0; j < data.size(); j++)
	{
		nodeNeighs[j] = (this->operator ()(nodeIndex,j) == 1) ? 1:0;
	}


	// go through matrix and sum suited elements (i.e. those [i,j] for which a_{nodeIndex,j} == 1)
	int sum = 0;
	int i = 0;
	int rlen = data.size();
	int indx = 0;

	while (indx < data.len())
	{
		// jump over non-interesting data.size()
		while (i < data.size() && nodeNeighs[i] == 0)
		{
			i++;
			indx += rlen;
			rlen--;
		}

		// for jumping over end
		if (i >= data.size())
			break;

		// go through elements of row
		for (int j = i; j < data.size(); j++)
		{
			// for nonexcluded and non-zero elements
			if (nodeNeighs[j] != 0 && data[indx] != 0)
			{
				sum += 2; // sum this and symmetric one
			}
			indx++;
		}

		// jump to next row
		i++;
		rlen--;
	}

	if (nodeNeighs != NULL)
		delete nodeNeighs;

	return sum;
}

int WeightedGraph::accessTriu(int *A, int i, int j)
{
	return (A == NULL) ? -1:((i <= j) ? (A[i*data.size() - (i*(i+1)/2) + j]):(A[j*data.size() - (j*(j+1)/2) + i]));
}


vector<BNode>* WeightedGraph::getPath(vector<BNode> *P, int i, int j)
{
	if (P == NULL)
	{
		P = new vector<BNode>;
		BNode fnode(0.0,0,i);
		P->push_back(fnode);
	}

	if (dist(i,j) == infinite)
		return NULL;
	int intermediate = accessTriu(next,i,j);
	if (intermediate == -1)
	{
		// there is an edge from i to j, with no vertices between
		BNode node(dist(i,j),i,j);
		P->push_back(node);
		return P;
	}
	else
	{
		getPath(P, i,intermediate);
		BNode last = P->back();
		BNode node(dist(last.node,intermediate),last.node,intermediate);
		//P->push_back(node);
		getPath(P, intermediate,j);
		return  P;
	}
}

SymMatrix<double>* WeightedGraph::distanceMatrix()
{
	if (D != NULL)
	{
		return D;
	}

	// there is never path longer than this
	infinite = std::numeric_limits<double>::infinity();

	// preparation work
	next = new int[data.len()];
	for (int i = 0; i < data.len(); i++)
		next[i] = -1;

	D = new SymMatrix<double>(data.size(),NULL);
	int rowsize = data.size();
	int j = 0;	// just indicator of already passed part of row - not an index
	for (int i = 0; i < data.len(); i++)
	{
		#if (INDICATION)
			cout << "\r Distance Matrix (prep):" << i << "/" << data.len();
			cout.flush();
		#endif

		// diagonal elements are not modified
		if (j != 0)
		{
			// weights are inverted and edges with no weights (0.0 exactly) have infitite distance
			if (data[i] == 0)
			{
				(*D)[i] = infinite;
			}
			else
			{
				(*D)[i] = data[i];  // we know now that on data[i] there is one
			}
		}
		else
		{
			(*D)[i] = 0.0;  // each vertex is in zero distance from itself
		}

		j++;
		if (j >= rowsize)
		{
			j = 0;
			rowsize--;
		}
	}

	int ij,ik,kj;
	int irow,jrow,krow;

	for (int k = 0; k < data.size(); k++)
	{
		krow = k*data.size() - (k*(k+1)/2);

		for (int i = 0; i < data.size(); i++)
		{
			irow = i*data.size() - (i*(i+1)/2);
			ik = (i <= k) ? irow + k:krow + i;

			for (int j = 0; j <= i; j++)
			{
				// Symmetric matrices have a_ij == a_ji
				jrow = j*data.size() - (j*(j+1)/2);
				//ij = (i <= j) ? irow + j:jrow + i;
				ij = jrow + i;
				//ji = jrow + i;
				kj = (k <= j) ? krow + j:jrow + k;

				if ((*D)[ij] > ((*D)[ik] + (*D)[kj]))
				{
					(*D)[ij] = (*D)[ik] + (*D)[kj];
					next[ij] = k;
				}
			}
		}
	}

	//cout << D->to_s() << endl;

	//printMatrix(D,10);

	return D;
}

double WeightedGraph::efficiency()
{
	double result = -1;
	distanceMatrix();

	// zero distances are only on diagonal
	double sum = 0.0;
	for (int i = 0; i < D->len(); i++) {
		double d = (*D)[i];
		// only for d < inf (i.e. < data.size()) and nonzero (for it add zero ;-))
		if (d < D->size() && d != 0)
			sum += 2 / d;

		#if (INDICATION)
			cout << "\r Efficiency:" << i << "/" << data.len();
			cout.flush();
		#endif
	}
	result = (1 / ((double) (D->size()) * (D->size() - 1))) * sum;

	return result;
}

double WeightedGraph::averageShortestPath()
{
	double result = -1;

	distanceMatrix();

	infinite = std::numeric_limits<double>::infinity();

	// test for existence of component
//	bool component = false;
//	for (int i = 0; i < D->len(); i++)
//	{
//		if ((*D)[i] > D->size())
//			component = true;
//
//		#if (INDICATION)
//			cout << "\r Component:" << i << "/" << D->len();
//			cout.flush();
//		#endif
//	}

	// do sum of a matrix - there are zeros on diagonal and both elements should be taken twice (triangular storage)
	double sum = 0;
	double n = 0;
	for (int i = 0; i < D->len(); i++)
	{
		double d = (*D)[i];
		if (d < infinite)
			sum += 2*d;
		else
			n++;

		#if (INDICATION)
			cout << "\r Average SP:" << sum << "/" << D->len();
			cout.flush();
		#endif
	}
	double divisor = ((double)D->size())*(D->size()-1) - n;
	result = (1/divisor)*sum;

	return result;
}


double WeightedGraph::clusteringCoefficient()
{
	// TODO
//	double sum = 0.0;
//
//	// specify node degrees
//	if (degrees == NULL)
//		nodeDegrees();
//
//	if (clusteringVector == NULL)
//	{
//		clusteringVector = new double[data.size()];
//
//		// count CC
//		for (int i = 0; i < data.size(); i++)
//		{
//			int k = degrees[i];
//			if (k >= 2)
//			{
//				double a = numerOfTriangles(i);
//				double ci = a/ (double) (k*k-k);
//				sum += ci;
//				clusteringVector[i] = ci;
//			}
//			else
//			{
//				clusteringVector[i] = 0;
//			}
//
//			#if (INDICATION)
//				cout << "\r Clustering coeficient:" << i << "/" << data.size();
//				cout.flush();
//			#endif
//		}
//
//		#if (INDICATION)
//			cout << endl;
//		#endif
//	}
//	else
//	{
//		for (int i = 0; i < data.size(); i++)
//			sum += clusteringVector[i];
//	}
//
//
//	double c = (1/ (double) data.size())*sum;
//
//	return c;
	return 0;
}

double* WeightedGraph::getClusteringCoefficientData()
{
	return clusteringVector;
}


void WeightedGraph::printMatrix(int max)
{
	cout << endl;
	for (int i = 0; i < max; i++)
	{
    	for (int j = 0; j < max; j++)
    	{
    		if (i <= j)
    			cout << (double) data[i*data.size() - (i*(i+1) / 2) + j];
    		else
    			cout << (double) data[j*data.size() - (j*(j+1) / 2) + i] << "*";
			cout << "\t ";
    	}
    	cout << endl;
	}
}

void WeightedGraph::printMatrix(double* A, int max)
{
	cout << endl;
	for (int i = 0; i < max; i++)
	{
    	for (int j = 0; j < max; j++)
    	{
    		if (i <= j)
    		{
    			double d = (double) A[i*data.size() - (i*(i+1) >> 1) + j];
   				cout << d;
    		}
    		else
    		{
    			double d = (double) A[j*data.size() - (j*(j+1) >> 1) + i];
   				cout << d;
    		}
			cout << "\t ";
    	}
    	cout << endl;
	}
}

void WeightedGraph::printTriangular()
{
	cout << endl;
	int k = 0;
	for (int i = 0; i < data.size(); i++)
	{
		for (int j = 0; j < i; j++)
		{
			cout << "\t ";
		}

    	for (int j = i; j < 10; j++)
    	{
    		cout << data[k++] << "\t ";
    	}
    	cout << endl;
	}
}

double WeightedGraph::deltaBetweeness()
{
	distanceMatrix();

	// test
//	vector<BNode> *path = NULL;
//	int a = 164;
//	int b = 6;
//	path = getPath(NULL, a, b);
//
//	double summ = 0.0;
//	cout << endl;
//	for (vector<BNode>::iterator it = path->begin(); it!=path->end(); ++it) {
//		BNode b = ((BNode) *it);
//		summ += b.dist;
//		cout << b.node << " (" << b.dist << ") -> ";
//	}
//	cout << " overall: "<< summ << " from d^w: "<< dist(a,b) << endl;


	// clear output
	long double* auxDeltaBetweenessVector = new long double[data.size()];
	long *sigma = new long[data.size()];
	bool *nu = new bool[data.size()];
	long* sigma_T = new long[1];
	for (int i = 0; i < data.size(); i++)
		auxDeltaBetweenessVector[i] = 0.0;

	// iterate over all vertices
	for (int s = 0; s < data.size(); s++)
	{
		// iterate over all resulting vertices that gives together all pairs {s,t}
		for (int t = s + 1; t < data.size(); t++)
		{
			// initiate
			for (int i = 0; i < data.size(); i++)
			{
				sigma[i] = 0.0;
				nu[i] = false;
				*sigma_T = 0;
			}
			vector<int> *P = new vector<int>;
			ASPData *adata = new ASPData(s,t,s,0,sigma,nu,P,sigma_T);

			#if (DEBUG)
				cout << "Cb for pair " << s << " " << t << " performing!" << endl;
			#endif

			ASP(adata, 0);

			if (*adata->sigma_T > 0)
			{
				for (int i = 0; i < data.size(); i++)
				{
					if (i != s && i != t)
						auxDeltaBetweenessVector[i] = auxDeltaBetweenessVector[i] + ((double) adata->sigma[i]/ (double) (*adata->sigma_T));
				}
			}
			else {
				 cout << " Warning: for a pair " << t << " : " << s << " there has been found no shortest path" << endl;
			}
			#if (INDICATION)
				cout << "Betweenness is " << endl;
				for (int j = 0; j < data.size(); j++)
					cout << auxDeltaBetweenessVector[j] << " ";
				cout << endl;
			#endif

			delete P;
			delete adata;
		}
	}

	delete sigma;
	delete nu;

	double sum = 0.0;
	deltaBetweenessVector = new double[data.size()];
	for (int i = 0; i < data.size(); i++)
	{
		deltaBetweenessVector[i] = 2*auxDeltaBetweenessVector[i];
		sum += deltaBetweenessVector[i];
	}
	double result = (1/((long double) data.size()))*sum;

	delete auxDeltaBetweenessVector;

	return result;
}

void WeightedGraph::ASP(ASPData *adata, int num)
{
	adata->nu[adata->u] = true;
	if (adata->u != adata->s)
		adata->P->push_back(adata->u);

	#if (INDICATION)
		outSpace(num);
		cout << "  performing vertex " << adata->u << endl;
	#endif

	// for all neighbors
	for (int v = 0; v < data.size(); v++)
	{
		if (adata->nu[v] == false)
		{
			// if there is an edge
			double weight = operator ()(adata->u, v);
			if (weight != 0.0)
			{
				double pathweight = adata->w + weight;

				// we reached terminal vertex
				if (v == adata->t)
				{
					#if (DEBUG)
						outSpace(num);
						cout << "  neighbor " << v << " is terminal and ";
					#endif

					if (pathweight - precisionEpsilon <= (1+weightDiffEpsilon)*dist(adata->s,adata->t))
					{
						#if (DEBUG)
							cout << " induces path of length " << pathweight << " : " << adata->s;
						#endif

						// update all vertices in P
						for (vector<int>::iterator it = adata->P->begin(); it!=adata->P->end(); ++it) {
							int pos = *it;
							adata->sigma[pos] = adata->sigma[pos] + 1;
							#if (DEBUG)
								cout << pos;
							#endif
						}
						#if (DEBUG)
							cout << adata->t << endl;
						#endif

						*adata->sigma_T = *adata->sigma_T + 1;
					}
					#if (DEBUG)
						cout << endl;
					#endif
				}
				else if (adata->nu[v] == false && pathweight + dist(v,adata->t) - precisionEpsilon <= (1+weightDiffEpsilon)*dist(adata->s,adata->t))
				{
					#if (DEBUG)
						outSpace(num);
						cout << "  neighbor " << v << " leads further" << endl;
					#endif
					ASPData* aa = adata->clone(v,pathweight);
					ASP(aa, num + 1);
					delete aa;
				}
				// else
				// {
				// 	#if (DEBUG)
				// 		outSpace(num);
				// 		cout << "  neighbor " << v << " induces no path (" << (pathweight + dist(v,adata->t)) << "=" << pathweight << "+" << dist(v,adata->t) << " < " << (1+weightDiffEpsilon)*dist(adata->s,adata->t) << ")" << endl;
				// 	#endif
				// }

				cout.flush();
			}
		}
	}
	adata->nu[adata->u] = false;
	adata->P->pop_back();
}

void WeightedGraph::outSpace(int k)
{
	for (int i = 0; i < k; i++)
		cout << " ";
}

long double WeightedGraph::betweeness()
{
	// clear output
	long double* auxBetweenessVector = new long double[data.size()];
	for (int i = 0; i < data.size(); i++)
		auxBetweenessVector[i] = 0.0;

	// create sigma, delta and P
	ULLONG_SUM_TYPE* sigma = new ULLONG_SUM_TYPE[data.size()];
	long double* delta = new long double[data.size()];
	list<int>* P = new list<int>[data.size()];

	// create and d and seen and their indication arrays
	long double* d = new long double[data.size()];
	long double* seen = new long double[data.size()];
	bool* ind_d = new bool[data.size()];
	bool* ind_seen = new bool[data.size()];


	// iterate over all vertices
	for (int s = 0; s < data.size(); s++)
	{

		#if (INDICATION)
			cout << "\r Betweeness:" << s << "/" << data.size();
			cout.flush();
		#endif

		stack<int> S;

		// clean arrays
		for (int j = 0; j < data.size(); j++)
		{
			ind_d[j] = false;
			ind_seen[j] = false;
			sigma[j] = 0;
			//d[j] = -1.0;
			//seen[j] = -1.0;
		}
		sigma[s] = 1;
		seen[s] = 0.0; ind_seen[s] = true;

		priority_queue <BNode, vector<BNode>, greater<vector<BNode>::value_type> > Q;
		//queue<BNode> Q;
		Q.push(BNode(0.0,s,s));

		while (!Q.empty())
		{
//			BNode node = Q.front();
			BNode node = Q.top();
			Q.pop();

			// test
//			cout << "step: <" << node.dist << " ," << node.predNode << " ," << node.node << ">" << endl;

			if (ind_d[node.node])
				continue;

			sigma[node.node] = sigma[node.node] + sigma[node.predNode];
			S.push(node.node);
			d[node.node] = node.dist;	ind_d[node.node] = true;

			// iterate over all neighbors
			int w = 0;
			while (w < data.size())
			{
				// find neighbor
				while (w < data.size() && this->operator ()(node.node,w) == 0)
					w++;
				// if no neigh exit
				if (w >= data.size())
					break;

				long double vwDist = d[node.node] + (long double) this->operator ()(node.node,w);

//				cout << "     > " << vwDist << " - " << seen[w] << " = " << (vwDist - seen[w]) << " " << endl;

				// test if w \nin d and (w \nin seen or vwDist < seen[w])
				if (!ind_d[w] && (!ind_seen[w] || vwDist < seen[w]))
				{
//					cout << "    Q.push(" << vwDist << " ," << node.node << " ," << w << ")" << endl;
					seen[w] = vwDist;	ind_seen[w] = true;
					Q.push(BNode(vwDist, node.node, w));
					sigma[w] = 0;
					P[w].clear();
					P[w].push_back(node.node);
				}
//				else if (fabs((double) vwDist - (double) seen[w]) < weightDiffEpsilon)
				else if (vwDist == seen[w])
				{
					sigma[w] = sigma[w] + sigma[node.node];
					P[w].push_back(node.node);
				}

				w++; // shift to next
			}


		}

		for (int j = 0; j < data.size(); j++)
			delta[j] = 0.0;

		#if (DEBUG)
			cout << "Sigmas:" << endl;
			for (int j = 0; j < data.size(); j++)
				cout << sigma[j] << " ";
			cout << endl;
		#endif

//		cout << "Sigmas (" << s << "):"<< endl;
//		for (int j = 0; j < data.size(); j++)
//			cout << sigma[j] << " ";
//		cout << endl;

		while (!S.empty())
		{
			int w = S.top();
			S.pop();
			list<int>::iterator it;

			#if (DEBUG)
				cout << "Prvek:" << w << endl;
			#endif

//			cout << endl << "Delta (" << w << "," << P[w].size() << "): ";
			for (it = P[w].begin() ; it != P[w].end(); it++)
			{
				int v = *it;  // int v = (int) *it
				delta[v] = delta[v] + ((long double)sigma[v]/(long double)sigma[w])*(1.0+delta[w]);

//				cout << delta[v] << " ";

				#if (DEBUG)
					cout << "\t "<< delta[v] << "=" << sigma[v] << "/" << sigma[w] << "*(1 + " << delta[w] << ")" << endl;
					cout << "\t Cb = "<< auxBetweenessVector[w] << endl;
				#endif
			}

			if (w != s)
			{
				auxBetweenessVector[w] = auxBetweenessVector[w] + delta[w];
			}
		}

		// delete aux data
		for (int i = 0;  i < data.size(); i++)
			P[i].clear();
	}

	delete[] P;
	delete sigma;
	delete d;
	delete delta;

	betweenessVector = new double[data.size()];
	for (int i = 0; i < data.size(); i++)
		betweenessVector[i] = (double) auxBetweenessVector[i];

	// TODO check these problems with long double

	long double sum = 0.0;
	for (int i = 0; i < data.size(); i++)
		sum += auxBetweenessVector[i];
	long double result = (1/((long double) data.size()))*sum;

	delete auxBetweenessVector;

	#if (DEBUG)
		cout << "Betweenness:" << endl;
		for (int j = 0; j < data.size(); j++)
			cout << betweenessVector[j] << " ";
		cout << endl;
	#endif

	return result;
}


double* WeightedGraph::getBetweenessData()
{
	if (betweenessVector == NULL)
	{
		betweeness();
	}

	betweenessGet = true;

	return betweenessVector;
}


double* WeightedGraph::getDeltaBetweenessData()
{
	if (deltaBetweenessVector == NULL)
	{
		deltaBetweeness();
	}

	deltaBetweenessGet = true;

	return deltaBetweenessVector;
}

