function [ BC ] = ndw_betweenness_bin(A,threshold)
%% [ BC ] = ndw_betweenness_bin(A,threshold)
% Betweenness centrality for binary matrices (graphs). Input parameters
% are adjacency matrix and threshold (it is applied to this matrix - so 
% matrix can even be weighted) or if threshold is not given the matrix 
% parameter is considered as binary.
%
% Parameters:
%       A ... adjacency matrix, possibly weighted - if threshold is given,
%             otherwise binary
%       threshold ... threshold that should be applied to this matrix. If
%                     not given matrix is considered as binary.
% Output:
%       BC ... shortest path betweenness centrality computed 
%
% Algorithm:
%       Brandes (2001) J Math Sociol 25:163-177
%
% Author: David Hartman
%         hartman@cs.cas.cz
%%
    if (nargin > 1)
        BC = ndw_betweenness(A,threshold);
    else
        BC = ndw_betweenness(A,0.5);

end

