#include <math.h>
#include <matrix.h>
#include <mex.h>
#include <stdio.h>
#include <string.h>
#include <limits>
#include "NetworksFunctionsInterface.h"

// matlab entry point
void retrieve_data( const mxArray* matptr, double*& data, int& dim1, int& dim2){
    // retrieve pointer from the MX form
    data = mxGetPr(matptr);
    
    // check that I actually received something
    if( data == NULL )
        mexErrMsgTxt("vararg{2} must be a [kxN] matrix of data\n");
    
    // retrieve amount of points
    dim1 = mxGetM(matptr);
    dim2 = mxGetN(matptr);
}

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[]){   
	// check input
	if( nrhs < 1 || !mxIsNumeric(prhs[0]) )
		mexErrMsgTxt("Correct arguments are matrix [NxN] and optionaly threshold [1].\n");
	
    // vector data by column - for symmetric matrix the same
    double* data;
    int dim1;
    int dim2;
    retrieve_data( prhs[0], data, dim1, dim2 );
    
    double charpath = 0;
    double* D = NULL;
    int* Di = NULL;
    // more parameters
    if (nrhs > 1)
    {
        // non-numeric
        if (!mxIsNumeric(prhs[1]))
        {
            char* par1 = mxArrayToString(prhs[1]);
            if (strcmp(par1,"dist") == 0)
            {
                // compute from distance matrix
                charpath = charpath_len_dist_weighted(data, dim1);
                D = data;
            }
            else
            {
                // error
                mexErrMsgTxt("Second parameter is not standard. Should be threshold or 'dist'.\n");
            }
            delete par1;
            
        }
        else
        {
            // extract threshold
            double* t = mxGetPr(prhs[1]);
            
            charpath = charpath_len_binary(data, dim1,*t, Di);
        }
    }
    else
    {
        // one parameter means adjacency matrix = weighted version
        charpath = charpath_len_weighted(data, dim1, D);
    }
    
    // copy results
    plhs[0] = mxCreateDoubleMatrix(1,1,mxREAL);
    double* pointer_to_result = mxGetPr(plhs[0]);
    pointer_to_result[ 0 ] = charpath;
    if (nlhs > 1)
    {
        plhs[1] = mxCreateDoubleMatrix(dim1,dim2,mxREAL);
        double* pointer_to_result2 = mxGetPr(plhs[1]);
        if (D == NULL)
        {
            for (int i = 0; i < dim1*dim2; i++)
                pointer_to_result2[ i ] = (Di[i] > dim1) ? mxGetInf():Di[i];
        }
        else
        {
            for (int i = 0; i < dim1*dim2; i++)
                pointer_to_result2[ i ] = (D[i] < std::numeric_limits<double>::infinity()) ? D[i]:mxGetInf();
        }
    }
}
