#include <cstring>
#include <math.h>
#include <matrix.h>
#include <mex.h>
#include "NetworksFunctionsInterface.h"

// matlab entry point
void retrieve_data( const mxArray* matptr, double*& data, int& dim1, int& dim2){
    // retrieve pointer from the MX form
    data = mxGetPr(matptr);
    
    // check that I actually received something
    if( data == NULL )
        mexErrMsgTxt("vararg{2} must be a [kxN] matrix of data\n");
    
    // retrieve amount of points
    dim1 = mxGetM(matptr);
    dim2 = mxGetN(matptr);
}

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[]){   
	// check input
	if( nrhs < 1 || !mxIsNumeric(prhs[0]) )
		mexErrMsgTxt("Correct arguments are matrix [NxN] and optionaly threshold [1].\n");
	
    // vector data by column - for symmetric matrix the same
    double* data;
    int dim1;
    int dim2;
    retrieve_data( prhs[0], data, dim1, dim2 );
    
    // get thresholds
    double* dBC = NULL;
    double* wDiff;
    if (nrhs > 1)
    {
        wDiff = mxGetPr(prhs[1]);
        dBC = delta_betweenness(data, dim1, *wDiff);
    }
    else
    {
		mexErrMsgTxt("No diff epsilon for delta betweenness given.\n");
    }
    
    
    // copy results
    if (dBC != NULL)
    {
        plhs[0] = mxCreateDoubleMatrix(dim1,1,mxREAL);
        double* pointer_to_result = mxGetPr(plhs[0]);
        memcpy(pointer_to_result,dBC,dim1*sizeof(double));

        delete dBC;
    }
}
