#include <cstring>
#include <math.h>
#include <matrix.h>
#include <mex.h>
#include <limits>
#include "NetworksFunctionsInterface.h"

// matlab entry point
void retrieve_data( const mxArray* matptr, double*& data, int& dim1, int& dim2){
    // retrieve pointer from the MX form
    data = mxGetPr(matptr);
    
    // check that I actually received something
    if( data == NULL )
        mexErrMsgTxt("vararg{2} must be a [kxN] matrix of data\n");
    
    // retrieve amount of points
    dim1 = mxGetM(matptr);
    dim2 = mxGetN(matptr);
}

void mexFunction(int nlhs, mxArray * plhs[], int nrhs, const mxArray * prhs[]){   
	// check input
	if( nrhs < 1 || !mxIsNumeric(prhs[0]) )
		mexErrMsgTxt("Correct arguments are matrix [NxN] and optionaly threshold [1].\n");
	
    // vector data by column - for symmetric matrix the same
    double* data;
    int dim1;
    int dim2;
    retrieve_data( prhs[0], data, dim1, dim2 );
    
    // get thresholds
    double* Dr = NULL;
    int* Di = NULL;
    if (nrhs > 1)
    {
        double* threshold = mxGetPr(prhs[1]);
        Di = distance_mat_binary(data, dim1, *threshold);
    }
    else
    {
        Dr = distance_mat_weighted(data, dim1);
    }
    
    // copy results
    plhs[0] = mxCreateDoubleMatrix(dim1,dim2,mxREAL);
    double* pointer_to_result = mxGetPr(plhs[0]);
    int len = dim1*dim2;
    if (Dr != NULL)
        memcpy(pointer_to_result,Dr,len*sizeof(double));
    else
        for (int i=0; i < len; i++)
            pointer_to_result[ i ] = (Di[i] > dim1) ? mxGetInf():Di[i];
            
    
    if (Dr != NULL)
        delete Dr;
    if (Di != NULL)
        delete Di;
    
}
